@@ -31,7 +31,7 @@ from calculate.lib.utils.device import (getUdevDeviceInfo,getDeviceType,
                         getUUIDDict,getCommonDeviceName)
 from calculate.install.variables.autopartition import Sizes
 from calculate.lib.utils.files import (listDirectory,pathJoin,readFile,FStab,
-                                       isMount)
+                                       isMount,getProgPath)
 from calculate.install.distr import PartitionDistributive
 from calculate.install.fs_manager import FileSystemManager
 
@@ -93,6 +93,7 @@ class VariableOsDeviceData(ReadonlyTableVariable):
     source = ['os_device_dev',
               'os_device_table',
               'os_device_type',
+              'os_device_ssd_set',
               'os_device_map',
               'os_device_name',
               'os_device_size']
@@ -109,8 +110,12 @@ class VariableOsDeviceDev(ReadonlyVariable,DeviceHelper):
     def get(self):
         """Get device /dev name"""
         if self.master is None:
-            self.master,slave = pty.openpty()
-            self.monitor = Popen(["/sbin/udevadm","monitor","--kernel",
+            try:
+                self.master, slave = pty.openpty()
+            except OSError:
+                raise VariableError('Failed to create PTY')
+            udevAdm = getProgPath('/sbin/udevadm')
+            self.monitor = Popen([udevAdm,"monitor","--kernel",
                                 "--subsystem-match=block"],stdout=slave,
                                 close_fds=True)
             os.close(slave)
@@ -196,6 +201,27 @@ class VariableOsDeviceMap(ReadonlyVariable):
         return map(lambda x:str(x[0]),
                enumerate(self.Get('os_device_dev')))
 
+class VariableOsDeviceSsdSet(ReadonlyVariable):
+    """
+    Table on device
+    """
+    type = "list"
+    ssd_names = ("SSD", "OCZ", "PLEXTOR")
+    udev_property = 'ID_ATA_ROTATION_RATE_RPM'
+
+    def get(self):
+        """Get device partition table"""
+        def isSsd(device, name):
+            prop = getUdevDeviceInfo(name=device)
+            rpm = prop.get(self.udev_property, None)
+            if rpm == "0" or any(x in name for x in self.ssd_names):
+                return "on"
+            else:
+                return "off"
+
+        return map(lambda x: isSsd(*x),
+                   zip(self.Get('os_device_dev'),
+                       self.Get('os_device_name')))
 
 class VariableOsDeviceTable(ReadonlyVariable):
     """
@@ -213,7 +239,12 @@ class VariableOsDeviceTable(ReadonlyVariable):
                  listDirectory(syspath))
         if childs:
             child = pathJoin(syspath,childs[0])
-            return getUdevDeviceInfo(path=child).get('ID_PART_ENTRY_SCHEME','')
+            udevinfo = getUdevDeviceInfo(path=child)
+            map_names = {'mbr':'dos',
+                         'msdos':'dos'}
+            table = (udevinfo.get('ID_PART_ENTRY_SCHEME','') or
+                        udevinfo.get('UDISKS_PARTITION_SCHEME',''))
+            return map_names.get(table,table)
         return ""
 
     def get(self):
@@ -1390,13 +1421,20 @@ class VariableOsInstallDiskOptions(ReadonlyVariable):
 
     def get(self):
         """TODO: get real options"""
+        def postProcess(opt):
+            l_opt = filter(None,opt.split(','))
+            if "noatime" in l_opt and not "nodiratime" in l_opt:
+                l_opt.append("nodiratime")
+            return ",".join(l_opt)
+                
         diskOpts = dict(filter(lambda x:x[1],
                         zip(self.Get('os_disk_dev'),
                             self.Get('os_disk_options'))))
-        return map(lambda x:diskOpts.get(x[0],x[1]),
+        return map(postProcess,
+               map(lambda x:diskOpts.get(x[0],x[1]),
                map(lambda x:(x[0],FileSystemManager.getDefaultOpt(x[1])),
                self.ZipVars('os_install_disk_dev',
-                            'os_install_disk_format')))
+                            'os_install_disk_format'))))
 
 class VariableOsInstallDiskFormat(ReadonlyVariable):
     """
@@ -1702,26 +1740,27 @@ class VariableOsInstallMbr(LocationHelper,Variable):
                     raise VariableError(_("You need a disk with a dos "
                                           "table for Flash install"))
             if tableOnBootDisk == "dos" and useBtrfs:
-                raise VariableError(_("You need a disk with a gpt "
-                                          "table for install on btrfs"))
+                raise VariableError(_("You need a disk with a GPT table for "
+                                      "install on a btrfs partition"))
             if rootType in ("usb-hdd","hdd") and tableOnBootDisk == "gpt":
                 efisize = self.Select('os_disk_size',
                            where=['os_disk_id','os_disk_parent'],
                            func=lambda os_disk_id,os_disk_parent:(
                            os_disk_id == 'EF02' and os_disk_parent == mbrDisk),
                            limit=1)
+                bios_grub_size = self.Get('cl_autopartition_bios_grub_size')
+                minsize = "%dMb" % (int(bios_grub_size) / Sizes.M)
                 if not efisize:
                     raise VariableError(
-                        _("Disk %s must have a 'bios_grub' partition")%mbrDisk)
+                        _("Your boot device must have a BIOS Boot partition "
+                          "({minsize})").format(minsize=minsize))
                 # проверка размера EF02 при установке на btrfs
                 elif useBtrfs:
-                    bios_grub_size = self.Get('cl_autopartition_bios_grub_size')
-                    if (efisize.isdigit() and bios_grub_size.isdigit() and 
+                    if (efisize.isdigit() and bios_grub_size.isdigit() and
                         int(efisize) < int(bios_grub_size)):
                         raise VariableError(
-                            _("Disk {diskname} must have a 'bios_grub' "
-                            "partition with size at least {size}Mb").format(
-                            diskname=mbrDisk,size=int(bios_grub_size)/Sizes.M))
+                            _("Your boot device must have a BIOS Boot "
+                              "partition ({minsize})").format(minsize=minsize))
         if value:
             if self.Get('os_grub2_path'):
                 self.checkForGrub2()
@@ -1817,6 +1856,20 @@ class VariableOsInstallRootDev(ReadonlyVariable):
         else:
             return self.Get('os_root_dev')
 
+class VariableOsInstallRootUuid(ReadonlyVariable):
+    def get(self):
+        """UUID корневого устройства"""
+        if self.Get('cl_action') == 'system':
+            root_dev = self.Get('os_install_root_dev')
+            return self.Select('os_install_disk_uuid',
+                               where='os_install_disk_dev',
+                               eq=root_dev,limit=1) or ''
+        else:
+            root_dev = self.Get('os_root_dev')
+            return self.Select('os_disk_uuid',
+                               where='os_disk_dev',
+                               eq=root_dev,limit=1) or ''
+
 class VariableOsInstallFstabMountConf(ReadonlyVariable,DeviceHelper):
     """
     FStab.conf contains for mount and bind points
@@ -1834,7 +1887,7 @@ class VariableOsInstallFstabMountConf(ReadonlyVariable,DeviceHelper):
                                        'os_install_disk_mount',
                                        'os_install_disk_format',
                                        'os_install_disk_options',
-                                       'os_disk_dev'],
+                                       'os_install_disk_dev'],
                                where='os_install_disk_mount',
                                func=lambda x:x[0] != "" and x[0] != "swap"),
                           lambda x,y: cmp(self.separateDevice(x[1]),
